package main

import (
	"bytes"
	"fmt"
	"github.com/tarm/serial"
	"log"
	"time"
)

func main() {
	if port, err := openComPort(2, 9600); err == nil {
		if resetCart(port) {
			log.Println("OK!")
		} else {
			log.Println("FAIL!")
		}
		log.Printf("0x%02X", readBytesCart(port, 0x0100, 0x4F))
	} else {
		log.Panicln(err)
	}
}

func openComPort(comNr, baud int) (*serial.Port, error) {
	c := &serial.Config{Name: fmt.Sprintf("COM%d", comNr), Baud: baud}
	s, err := serial.OpenPort(c)
	return s, err
}

func changeBaud(comNr, baud int) {
	// Check if Port is already opened
	s, err := serial.OpenPort(&serial.Config{Name: fmt.Sprintf("COM%d", comNr)})

	if err != nil {
		// Close Port first
		if err := s.Close(); err != nil {
			openComPort(comNr, baud)
		} else {
			panic(err)
		}
	}
	// Do nothing in case the port isn't opened
}

func enumerateComPorts() []string {
	var ports []string
	for i := 0; i < 256+1; i++ {
		s, err := openComPort(i, 9600)
		if err == nil {
			if err := s.Close(); err == nil {
				ports = append(ports, fmt.Sprintf("COM%d", i))
			} else {
				panic(err)
			}
		}
	}
	return ports
}

func writeSerial(port *serial.Port, buf []byte) {
	if _, err := port.Write(buf); err != nil {
		panic(err)
	}
}

func readSerial(port *serial.Port, bufferSize int) []byte {
	buf := make([]byte, bufferSize)
	_, err := port.Read(buf)
	if err != nil {
		panic(err)
	}
	buf = bytes.TrimRight(buf, "\x00")

	return buf
}

func readResponse(port *serial.Port, bufferSize int, timeToWait time.Duration) []byte {
	waitTimer := time.NewTimer(timeToWait)
	<-waitTimer.C
	waitTimer.Stop()
	return readSerial(port, 10)
}

func resetCart(port *serial.Port) bool {
	writeSerial(port, []byte{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}) // Write a few zeroes just in case previously a command got executed and interrupted
	time.Sleep(time.Second)
	if string(readSerial(port, 5)) == "READY" {
		return true
	}
	return false
}

func makeWordNibbles(word uint16) (byte, byte) {
	h := byte((word & 0xFF00) >> 8)
	l := byte(word & 0x00FF)
	return h, l
}

func readByteCart(port *serial.Port, address uint16) byte {
	h, l := makeWordNibbles(address)
	writeSerial(port, []byte{0x03, h, l})
	return readSerial(port, 1)[0]
}

func writeByteCart(port *serial.Port, address uint16, value byte) {
	h, l := makeWordNibbles(address)
	writeSerial(port, []byte{0x04, h, l, value})
}

func readBytesCart(port *serial.Port, address, length uint16) []byte {
	hAddr, lAddr := makeWordNibbles(address)
	hLen, lLen := makeWordNibbles(length)
	writeSerial(port, []byte{0x05, hAddr, lAddr, hLen, lLen})
	time.Sleep(time.Second)
	return readSerial(port, int(length))
	//return readResponse(port, int(length), 250*time.Millisecond)
}
